@@ -1,18 +1,24 @@
 import imp
+import logging
 import pkgutil
 import sys
 import traceback
+
 import requests
 
-from streamlink.utils import update_scheme
+from streamlink.logger import StreamlinkLogger, Logger
+from streamlink.utils import update_scheme, memoize
 from streamlink.utils.l10n import Localization
 from . import plugins, __version__
 from .compat import is_win32
 from .exceptions import NoPluginError, PluginError
-from .logger import Logger
 from .options import Options
 from .plugin import api
 
+# Ensure that the Logger class returned is Streamslink's for using the API (for backwards compatibility)
+logging.setLoggerClass(StreamlinkLogger)
+log = logging.getLogger(__name__)
+
 
 def print_small_exception(start_after):
     type, value, traceback_ = sys.exc_info()
@@ -71,8 +77,19 @@ class Streamlink(object):
             "locale": None
         })
         self.plugins = {}
-        self.logger = Logger()
         self.load_builtin_plugins()
+        self._logger = None
+
+
+    @property
+    def logger(self):
+        """
+        Backwards compatible logger property
+        :return: Logger instance
+        """
+        if not self._logger:
+            self._logger = Logger()
+        return self._logger
 
     def set_option(self, key, value):
         """Sets general options used by plugins and streams originating
@@ -338,7 +355,6 @@ class Streamlink(object):
         :param level: level of logging to output
 
         """
-
         self.logger.set_level(level)
 
     def set_logoutput(self, output):
@@ -349,6 +365,7 @@ class Streamlink(object):
         """
         self.logger.set_output(output)
 
+    @memoize
     def resolve_url(self, url, follow_redirect=True):
         """Attempts to find a plugin that can use this URL.
 
@@ -426,12 +443,13 @@ class Streamlink(object):
         :param path: full path to a directory where to look for plugins
 
         """
-
         for loader, name, ispkg in pkgutil.iter_modules([path]):
             file, pathname, desc = imp.find_module(name, [path])
+            # set the full plugin module name
+            module_name = "streamlink.plugin.{0}".format(name)
 
             try:
-                self.load_plugin(name, file, pathname, desc)
+                self.load_plugin(module_name, file, pathname, desc)
             except Exception:
                 sys.stderr.write("Failed to load plugin {0}:\n".format(name))
                 print_small_exception("load_plugin")
@@ -441,13 +459,18 @@ class Streamlink(object):
     def load_plugin(self, name, file, pathname, desc):
         # Set the global http session for this plugin
         api.http = self.http
+
         module = imp.load_module(name, file, pathname, desc)
 
         if hasattr(module, "__plugin__"):
             module_name = getattr(module, "__name__")
+            plugin_name = module_name.split(".")[-1]  # get the plugin part of the module name
 
             plugin = getattr(module, "__plugin__")
-            plugin.bind(self, module_name)
+            plugin.bind(self, plugin_name)
+
+            if plugin.module in self.plugins:
+                log.debug("Plugin {0} is being overridden by {1}".format(plugin.module, pathname))
 
             self.plugins[plugin.module] = plugin
 
